# -*- coding: utf-8 -*-
# Static constants for dealing with different langauge wikis

# List of wikipedia prefixes - not exhaustive
prefixes = ['ru', 'nap', 'ja', 'ta', 'tl', 'az', 'ht', 'et', 'ca', 'wa', 'pms', 
'de', 'nl', 'su', 'bpy', 'fa', 'tr', 'en', 'eu', 'io', 'jv', 'nds', 'pt', 'zh', 
'bn', 'lt', 'la', 'nn', 'no', 'he', 'sh', 'lb', 'ro', 'hr', 'scn', 'da', 'it', 
'te', 'br', 'an', 'fr', 'cs', 'is', 'new', 'lv', 'ast', 'uk', 'mk', 'oc', 'ku', 
'sl', 'sk', 'af', 'ka', 'pl', 'ceb', 'th', 'sv', 'id', 'bs', 'hi', 'el', 'cy', 
'gl', 'vi', 'sq', 'fi', 'mr', 'hu', 'ar', 'bg', 'ms', 'ko', 'be']

# Keyword for identifying category labels in different languages
# TODO: Complete this list
category_identifier =\
  { 'en' : 'Category'
  , 'fr' : 'Catégorie'
  , 'de' : 'Kategorie'
  , 'ja' : 'Category'
  , 'it' : 'Categoria'
  , 'ru' : 'Категория'
  , 'pl' : 'Kategoria'
  , 'nl' : 'Categorie'
  , 'sv' : 'Kategori'
  , 'zh' : 'Category'
  }

